@@ -28,7 +28,7 @@ extern uint32   export_password_hash;
 
 extern unsigned char password[PASSWORD_MAXLEN+1];
 
-void		exports_parse(void);
+int		exports_parse(void);
 int		exports_options(const char *path, struct svc_req *rqstp, char **password, uint32 *fsid);
 int             export_point(const char *path);
 char            *export_point_from_fsid(uint32 fsid, time_t **last_mtime, uint32 **dir_hash);
