@@ -13,10 +13,12 @@
 #include <limits.h>
 #include <stdlib.h>
 #include <string.h>
+#ifndef WIN32
 #include <syslog.h>
 #include <unistd.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+#endif				       /* WIN32 */
 #include <fcntl.h>
 
 #include "nfs.h"
@@ -57,18 +59,22 @@ static void add_mount(const char *path, struct svc_req *rqstp)
     char *host;
 
     new = malloc(sizeof(struct mountbody));
-    if (!new)
+    if (!new) {
+	logmsg(LOG_CRIT, "add_mount: Unable to allocate memory");
 	return;
+    }
 
     host = inet_ntoa(get_remote(rqstp));
     new->ml_hostname = malloc(strlen(host) + 1);
     if (!new->ml_hostname) {
+	logmsg(LOG_CRIT, "add_mount: Unable to allocate memory");
 	free(new);
 	return;
     }
 
     new->ml_directory = malloc(strlen(path) + 1);
     if (!new->ml_directory) {
+	logmsg(LOG_CRIT, "add_mount: Unable to allocate memory");
 	free(new->ml_hostname);
 	free(new);
 	return;
@@ -157,12 +163,16 @@ mountres3 *mountproc_mnt_3_svc(dirpath * argp, struct svc_req * rqstp)
 
     /* Check for "mount commands" */
     if (strncmp(dpath, "@getnonce", sizeof("@getnonce") - 1) == 0) {
-	gen_nonce(nonce);
-	result.fhs_status = MNT3_OK;
-	result.mountres3_u.mountinfo.fhandle.fhandle3_len = 32;
-	result.mountres3_u.mountinfo.fhandle.fhandle3_val = nonce;
-	result.mountres3_u.mountinfo.auth_flavors.auth_flavors_len = 1;
-	result.mountres3_u.mountinfo.auth_flavors.auth_flavors_val = &auth;
+	if (backend_gen_nonce(nonce) < 0) {
+	    result.fhs_status = MNT3ERR_IO;
+	} else {
+	    result.fhs_status = MNT3_OK;
+	    result.mountres3_u.mountinfo.fhandle.fhandle3_len = 32;
+	    result.mountres3_u.mountinfo.fhandle.fhandle3_val = nonce;
+	    result.mountres3_u.mountinfo.auth_flavors.auth_flavors_len = 1;
+	    result.mountres3_u.mountinfo.auth_flavors.auth_flavors_val =
+		&auth;
+	}
 	return &result;
     } else if (strncmp(dpath, "@password:", sizeof("@password:") - 1) == 0) {
 	char pw[PASSWORD_MAXLEN + 1];
@@ -177,7 +187,7 @@ mountres3 *mountproc_mnt_3_svc(dirpath * argp, struct svc_req * rqstp)
 	char otp[PASSWORD_MAXLEN + 1];
 
 	/* Our calculated otp */
-	unsigned char hexdigest[32];
+	char hexdigest[32];
 
 	mnt_cmd_argument(&dpath, "@otp:", otp, PASSWORD_MAXLEN);
 	if (exports_options(dpath, rqstp, &password, NULL) != -1) {
@@ -187,7 +197,7 @@ mountres3 *mountproc_mnt_3_svc(dirpath * argp, struct svc_req * rqstp)
 	    authenticated = !strncmp(hexdigest, otp, 32);
 
 	    /* Change nonce */
-	    gen_nonce(nonce);
+	    backend_gen_nonce(nonce);
 	}
 	/* else leave authenticated unchanged */
     }
@@ -211,8 +221,9 @@ mountres3 *mountproc_mnt_3_svc(dirpath * argp, struct svc_req * rqstp)
 
     if ((exports_options(buf, rqstp, &password, NULL) == -1)
 	|| (!authenticated && password[0])
-	|| (!IS_SECURE(ntohs(get_port(rqstp))))
-       ) {
+	|| (!(exports_opts & OPT_INSECURE) &&
+	    !IS_SECURE(ntohs(get_port(rqstp))))
+	) {
 	/* not exported to this host or at all, or a password defined and not 
 	   authenticated */
 	result.fhs_status = MNT3ERR_ACCES;
